module PuppetX
  module EnterpriseModules
    # Docs
    module OraInstall
      # Docs
      module GetHomes

        def default_database_sid
          Hash[*oratab_content.scan(/^(?!#)(.*)\:(.*)\:.*$/).flatten].keys.first
        end

        def oracle_home_for(sid)
          Hash[*oratab_content.scan(/^(?!#)(.*)\:(.*)\:.*$/).flatten].select { |k, _v| k == sid }.values.first
        end

        def user_for_file(home)
          #
          # To run opatch, we need the owner of the grid and oracle directory. The mechanism
          # we use to get this user, is to fetch the owner of the bin directory in the grid or
          # oracle home. Sometimes however, these directories also contain files with root as owner.
          # since root can never be the user of the grid or oracle processen, we skip those.
          #
          a_file = first_non_root_file(home)
          if a_file
            uid = File.stat("#{home}/#{a_file}").uid
            Etc.getpwuid(uid).name
          else
            nil
          end
        end

        def first_non_root_file(dir)
          if Dir.exist?(dir)
            Dir.new(dir).each { |file| break file if File.stat("#{dir}/#{file}").uid.nonzero? }
          else
            nil
          end
        end

        def oracle_homes(resources = nil)
          if resources
            resources.map { |k, v| v.oracle_product_home_dir }.uniq
          else
            oratab_content.scan(/^(?!#).*\:(.*)\:.*$/).flatten.uniq
          end
        end

        def installed_patches
          oracle_homes.collect do |oracle_home|
            patches_in_home(oracle_home, ora_invdir)
          end.flatten
        end

        def patches_in_home(oracle_product_home_dir, orainst_dir)
          os_user = user_for_file(oracle_product_home_dir)
          if os_user
            Dir.chdir(oracle_product_home_dir) do
              full_command = "#{oracle_product_home_dir}/OPatch/opatch lspatches -oh #{oracle_product_home_dir} -invPtrLoc #{orainst_dir}/oraInst.loc"
              raw_list = Puppet::Util::Execution.execute(full_command, :failonfail => true, :uid => os_user, :combine => true )
              Puppet.debug "\n#{raw_list}"
              cleanup_opatch_residue([oracle_product_home_dir])
              raw_list.scan(/^(\d+);(.*)$/).collect { |e| { :name => "#{oracle_product_home_dir}:#{e[0]}", :comment => e[1] } }
            end
          else
            {}
          end
        end

        def cleanup_opatch_residue(homes)
          # TODO: This is duplicate code from facts.rb. We need to make this more common
          # Some older versions of Opatch leave files and directories arround
          # When running opatch often, this causes some issues. This is to make
          # sure all files created by running Opatch as part of the fact, are remopved
          basename = Time.now.strftime("%Y-%m-%d_%H-%M-*%p")
          FileUtils.rm_f("/OraInstall2018#{basename}")
          homes.each {|home| FileUtils.rm_f("#{home}/cfgtoollogs/opatch/lsinv/lsinventory#{basename}.txt") }
        end

        def oratab_content
          File.read(oratab_file)
        end

        def oratab_file
          "#{ora_invdir}/oratab"
        end

        def ora_invdir
          os = Facter.value(:kernel)
          case os
          when 'Linux' then
            '/etc'
          when 'SunOS' then
            '/var/opt/oracle'
          else
            '/etc'
          end
        end
      end
    end
  end
end
